;//////////////////////////////////////////////////////
;       SVGAdem2.asm 
;//////////////////////////////////////////////////////


SCREEN          EQU     0A0000h
FIXUP           EQU     12345678h


_DATA   SEGMENT  'DATA' USE32

extrn _Curwind  :WORD                     ;curently mapped svga page (read-write)
extrn _Addgran  :WORD                     ;window granularity        (read only)


_DATA   ENDS

_TEXT   SEGMENT PUBLIC 'CODE' USE32
public  AddBlit_,SubBlit_,BlitMemVram_

; all the routines are hard coded 

;------------------------------------------------------------------
;------------------------------------------------------------------
;       BlitMemVram
;
;       Blits data from ram to the screen in SVGA mode (640x480x256)
;       input: esi=source(width assumed to be 768),ebx=screen x, eax=screen y ,ecx=width, edi=height
;       output: none
;        
;
;------------------------------------------------------------------
BlitMemVram_    PROC    Near
        pusha
        mov     dx,3dah
s2o:    in      al,dx
        test    al,8
        jz      s2o
        popa
        push    edi                 
        mov     di,640                          
        mul     di                      ;get page number
        pop     edi
                
        add     ax,bx                   ;now ax=offset, dx=page #                                        
        adc     dx,0
                

        cmp     dx,_Curwind             ; already in right page?
        jz      okay_copy                     
        
        pusha                           ;if not - then set new page
        mov     eax,edx
        mov     _Curwind,ax     
        imul    word ptr _Addgran       ;page#=granularity * page#
        mov     dx,ax                   
        xor     ebx,ebx   
        mov     eax,04f05h                       
        int     10h                     ;call VESA BIOS          
        popa
        
okay_copy:
                
        push    ecx                     ;if !(width/4) then figure out remainder 
        mov     ebx,ecx                          
        shr     ecx,2        
        and     ebx,11b                
        mov     DWORD PTR fix@5-4,ecx   ;how many stosd (main loop:self-modifying code)                 
        pop     ecx
        
        mov     edx,edi                 ;dx = height        
        mov     edi,SCREEN              ;edi pointing to video memory
        add     edi,eax                 ;+ starting offset
        
        jmp     $+2
        cld
main_loop:
        push    ecx                     ;save width        
        push    edi                     ;save pointer to the screen 
        
        mov     eax,edi               
        add     ax,cx                   ;check if line is crossing page
        jc      cros_page 
        
        mov     ecx,FIXUP               ;how many dwords 
fix@5:
        rep     movsd                   ;copy dwords
        mov     ecx,ebx        
        rep     movsb                   ;and remainder
next_line_copy:               
        
                                        
        
        pop     edi
        pop     ecx                     ;restore width
        add     esi,128                   
        dec     dx                      ;done?
        jz      exit_copy                   
        adc     di,640                  ;next line        
        jnc     main_loop               ;crossing page?
        
        pusha
        inc     word ptr _Curwind       ;next page
        mov     ax,_Curwind             ;
        imul    word ptr _Addgran       ;granularity         
        xor     ebx,ebx
        mov     dx,ax
        mov     eax,04f05h        
        int     10h                     ;call VESA BIOS
        popa        
        jmp     main_loop               ;do everything again 
                                  
cros_page:   
        mov     ecx,eax                 ;cx=# of pixels in new page
                         
more_pixels:  
        
        mov     al,[esi]                ;copy pixel to vram      
        mov     [edi],al
        inc     esi                       
        add     di,1
        jnc     more_pixels
        and     ecx,ecx                 ;something left for next page?
        jz      next_line_copy
                            
        pusha
        inc     word ptr _Curwind       ;next page
        mov     ax,_Curwind             ;
        imul    word ptr _Addgran       ;granularity         
        xor     ebx,ebx
        mov     dx,ax
        mov     eax,04f05h        
        int     10h                     ;call VESA BIOS
        popa
        
more_pixels2:
        mov     al,[esi]
        mov     [edi],al                ;copy remaining part to new page
        inc     esi
        inc     di
        dec     cx
        jnz     more_pixels2
        
        
        add     esi,128                                       
        pop     edi
        pop     ecx                     ;restore width        
        add     di,640                  ;next line (can't be next page - we just crossed it!)
        dec     dx                      ;height - are we done?
        jnz     main_loop                  
                                                   
exit_copy:
        ret
BlitMemVram_    ENDP
;------------------------------------------------------------------
;------------------------------------------------------------------
;       AddBlit
;
;       Add transparent windows to the VScreen buffer  
;       input: esi=mask buffer(256x200),edi=screen buffer (768x640)
;       output: none
;              
;------------------------------------------------------------------

AddBlit_        Proc    NEAR

        mov     edx,200
loopik2:        
        mov     ecx,(256)/32
loopik:        
        mov     eax,[esi]
        or      [edi],eax      ; unroll the loop ( 32 pixels at a time )                 
        add     edi,4
        add     esi,4
        mov     eax,[esi]
        or      [edi],eax        
        add     esi,4  
        add     edi,4     
        mov     eax,[esi]
        or      [edi],eax        
        add     esi,4  
        add     edi,4
        mov     eax,[esi]
        or      [edi],eax        
        add     esi,4  
        add     edi,4                     
        mov     eax,[esi]
        or      [edi],eax                       
        add     edi,4
        add     esi,4
        mov     eax,[esi]
        or      [edi],eax        
        add     esi,4  
        add     edi,4     
        mov     eax,[esi]
        or      [edi],eax        
        add     esi,4  
        add     edi,4
        mov     eax,[esi]
        or      [edi],eax        
        add     esi,4  
        add     edi,4                     
                                                     
        dec     ecx
        jnz     loopik
        add     edi,768-256
        dec     edx
        jnz     loopik2
        ret
AddBlit_        Endp
;------------------------------------------------------------------
;------------------------------------------------------------------
;       SubBlit
;
;       Restore contents of the VScreen buffer  
;       input: edi=VScreen buffer 
;       output: none
;              
;------------------------------------------------------------------

SubBlit_        PROC NEAR
        
        
      
        mov     ebx,0f0f0f0fh
        mov     edx,200
petl2:        
        
        
        mov     ecx,256/32
pel1:        
        and     [edi],ebx 
        add     edi,4             
        and     [edi],ebx
        add     edi,4
        and     [edi],ebx 
        add     edi,4             
        and     [edi],ebx
        add     edi,4
        and     [edi],ebx 
        add     edi,4             
        and     [edi],ebx
        add     edi,4
        and     [edi],ebx 
        add     edi,4             
        and     [edi],ebx
        add     edi,4
                
        dec     ecx
        jnz     pel1
        add     edi,768-256
        dec     edx
        jnz     petl2
        ret
SubBlit_        endp
      

_TEXT   ENDS
        
        END

 
