;//////////////////////////////////////////////////////
;       SVGAdemo.asm 
;//////////////////////////////////////////////////////


ANGLE360        EQU     256
ANGLEMASK       EQU     (ANGLE360 - 1)
ANGLE180        EQU     (ANGLE360 / 2)
ANGLE90         EQU     (ANGLE360 / 4)
SCREEN_WIDTH    EQU     320
SCREEN_HEIGHT   EQU     240
SCREEN_CENTERX  EQU     (SCREEN_WIDTH / 2)
SCREEN_CENTERY  EQU     (SCREEN_HEIGHT / 2)
SOURCE_SIZE     EQU     64
SOURCE_SBITS    EQU     6
SCREEN          EQU     0A0000h
FIXUP           EQU     12345678h

_DATA   SEGMENT  'DATA' USE32

extrn _Curwind  :WORD                     ;curently mapped svga page (read-write)
extrn _Addgran  :WORD                     ;window granularity        (read only)
extern _spointer:DWORD


startingU       dd      ?
startingV       dd      ?
dUCol           dd      ?                 ;deltas
dVCol           dd      ?
dURow           dd      ?
dVRow           dd      ?                       
rowCount        dd      ?
angle           dd      ?
bscreen         dd      ?                 ;screen buffer (320x240) 
bitmap          dd      ?
scale           dd      ?

_DATA   ENDS

_TEXT   SEGMENT PUBLIC 'CODE' USE32
public  RotateBlit_,AddBlit_,SubBlit_,BlitMemVram_

;------------------------------------------------------------------
;------------------------------------------------------------------
;       BlitMemVram
;
;       Blits data from ram to the screen in SVGA mode (640x480x256)
;       input: esi=source ,ebx=screen x, eax=screen y ,ecx=width, edi=height
;       output: none
;        
;
;------------------------------------------------------------------
BlitMemVram_    PROC    Near
        pusha
        mov     dx,3dah
s2o:    in      al,dx
        test    al,8
        jz      s2o
        popa
        push    edi                 
        mov     di,640                          
        mul     di                      ;get page number
        pop     edi
                
        add     ax,bx                   ;now ax=offset, dx=page #                                        
        adc     dx,0
                

        cmp     dx,_Curwind             ; already in right page?
        jz      okay_copy                     
        
        pusha                           ;if not - then set new page
        mov     eax,edx
        mov     _Curwind,ax     
        imul    word ptr _Addgran       ;page#=granularity * page#
        mov     dx,ax                   
        xor     ebx,ebx   
        mov     eax,04f05h                       
        int     10h                     ;call VESA BIOS          
        popa
        
okay_copy:
                
        push    ecx                     ;if !(width/4) then figure out remainder 
        mov     ebx,ecx                          
        shr     ecx,2        
        and     ebx,11b                
        mov     DWORD PTR fix@5-4,ecx   ;how many stosd (main loop:self-modifying code)                 
        pop     ecx
        
        mov     edx,edi                 ;dx = height        
        mov     edi,SCREEN              ;edi pointing to video memory
        add     edi,eax                 ;+ starting offset
        
        jmp     $+2
        cld
main_loop:
        push    ecx                     ;save width        
        push    edi                     ;save pointer to the screen 
        
        mov     eax,edi               
        add     ax,cx                   ;check if line is crossing page
        jc      cros_page 
        
        mov     ecx,FIXUP               ;how many dwords 
fix@5:
        rep     movsd                   ;copy dwords
        mov     ecx,ebx        
        rep     movsb                   ;and remainder
                                        
next_line_copy:               
        
        pop     edi
        pop     ecx                     ;restore width                   
        dec     dx                      ;done?
        jz      exit_copy                   
        add     di,640                  ;next line        
        jnc     main_loop               ;crossing page?
        
        pusha
        inc     word ptr _Curwind       ;next page
        mov     ax,_Curwind             ;
        imul    word ptr _Addgran       ;granularity         
        xor     ebx,ebx
        mov     dx,ax
        mov     eax,04f05h        
        int     10h                     ;call VESA BIOS
        popa        
        jmp     main_loop               ;do everything again 
                                  
cros_page:   
        mov     ecx,eax                 ;cx=# of pixels in new page
                         
more_pixels:  
        mov     al,[esi]                ;copy pixel to vram      
        mov     [edi],al
        inc     esi                       
        add     di,1
        jnc     more_pixels
        and     ecx,ecx                 ;something left for next page?
        jz      next_line_copy
                            
        pusha
        inc     word ptr _Curwind       ;next page
        mov     ax,_Curwind             ;
        imul    word ptr _Addgran       ;granularity         
        xor     ebx,ebx
        mov     dx,ax
        mov     eax,04f05h        
        int     10h                     ;call VESA BIOS
        popa
        
more_pixels2:
        mov     al,[esi]
        mov     [edi],al                ;copy remaining part to new page
        inc     esi
        inc     di
        dec     cx
        jnz     more_pixels2
                                       
        pop     edi
        pop     ecx                     ;restore width        
        add     di,640                  ;next line (can't be next page - we just crossed it!)
        dec     dx                      ;height - are we done?
        jnz     main_loop                  
                                                   
exit_copy:
        ret
BlitMemVram_    ENDP
;------------------------------------------------------------------
;------------------------------------------------------------------
;       RotateBlit
;
;       Scales and rotates bitmap in ram buffer 
;       input: esi=bitmap,eax=angle,edx=scale,edi=buffer
;       output: none
;       :slighlty modified version of a routine written by Peter Freese
;       :"More Tricks of The Game Programing Gurus"
;------------------------------------------------------------------
      
RotateBlit_     PROC    NEAR


; calculate horizontal deltas

        pusha
        mov     [angle],eax
        mov     [scale],edx
        mov     [bitmap],esi
        mov     [bscreen],edi
        mov     eax,[angle]
        add     eax,ANGLE90
        and     eax,ANGLEMASK
        mov     esi,_spointer
        mov     eax,[esi + eax * 4]
        shl     eax,16 - SOURCE_SBITS
        imul    scale
        shrd    eax,edx,16
        mov     [dUCol],eax
        mov     DWORD PTR @@fdUCol1-4,eax
        mov     DWORD PTR @@fdUCol2-4,eax
        mov     DWORD PTR @@fdUCol3-4,eax
        mov     DWORD PTR @@fdUCol4-4,eax
        mov     DWORD PTR @@fdUCol5-4,eax
        mov     DWORD PTR @@fdUCol6-4,eax
        mov     DWORD PTR @@fdUCol7-4,eax
        mov     DWORD PTR @@fdUCol8-4,eax

        mov     eax,[angle]
        and     eax,ANGLEMASK
        mov     esi,_spointer
        mov     eax,[esi + eax * 4]
        shl     eax,16 - SOURCE_SBITS
        imul    [scale]
        shrd    eax,edx,16
        mov     [dVCol],eax
        mov     DWORD PTR @@fdVCol1-4,eax
        mov     DWORD PTR @@fdVCol2-4,eax
        mov     DWORD PTR @@fdVCol3-4,eax
        mov     DWORD PTR @@fdVCol4-4,eax
        mov     DWORD PTR @@fdVCol5-4,eax
        mov     DWORD PTR @@fdVCol6-4,eax
        mov     DWORD PTR @@fdVCol7-4,eax
        mov     DWORD PTR @@fdVCol8-4,eax

; calculate vertical deltas

        mov     eax,[dVCol]
        neg     eax                
        mov     [dURow],eax

        mov     eax,[dUCol]  
        mov     [dVRow],eax

        mov     eax,80000000h
        mov     [startingU],eax
        mov     [startingV],eax

; move up by yOrg

        mov     eax,[dUCol]
        imul    eax,SCREEN_CENTERX
        sub     [startingU],eax

        mov     eax,[dURow]
        imul    eax,SCREEN_CENTERY
        sub     [startingU],eax

        mov     eax,[dVCol]
        imul    eax,SCREEN_CENTERX
        sub     [startingV],eax

        mov     eax,[dVRow]
        imul    eax,SCREEN_CENTERY
        sub     [startingV],eax

; fixup end of row deltas

        mov     eax,[dUCol]
        imul    eax,SCREEN_WIDTH
        neg     eax
        add     eax,[dURow]
        mov     DWORD PTR @@fdURow-4,eax

        mov     eax,[dVCol]
        imul    eax,SCREEN_WIDTH
        neg     eax
        add     eax,[dVRow]
        mov     DWORD PTR @@fdVRow-4,eax

        mov     esi,bitmap
        mov     edi,bscreen

        mov     ecx,[startingU]
        mov     edx,[startingV]

        mov     eax,SCREEN_HEIGHT               ; initialize row count
        mov     [rowCount],eax

        jmp     $+2
;----------------------------------------------- row loop            
@@rowloop:
        push    ebp
        mov     ebp,SCREEN_WIDTH / 8            ; initialize column count
        
;----------------------------------------------- column loop (inner loop) 

@@colloop:                                      ; unrolled 8 times
        mov     ebx,edx
        shr     ebx,32 - SOURCE_SBITS
        shld    ebx,ecx,SOURCE_SBITS
        add     edx,FIXUP
@@fdVCol1:
        add     ecx,FIXUP
@@fdUCol1:
        mov     al,[esi+ebx]        
        mov     ebx,edx
        shr     ebx,32 - SOURCE_SBITS
        shld    ebx,ecx,SOURCE_SBITS
        add     edx,FIXUP
@@fdVCol2:
        add     ecx,FIXUP
@@fdUCol2:
        mov     ah,[esi+ebx]

        mov     ebx,edx
        shl     eax,16
        
        shr     ebx,32 - SOURCE_SBITS
        shld    ebx,ecx,SOURCE_SBITS
        add     edx,FIXUP
@@fdVCol3:
        mov     al,[esi+ebx]
        add     ecx,FIXUP
@@fdUCol3:

        mov     ebx,edx
        
        shr     ebx,32 - SOURCE_SBITS
        shld    ebx,ecx,SOURCE_SBITS
        add     edx,FIXUP
@@fdVCol4:
        mov     ah,[esi+ebx]
        add     ecx,FIXUP
@@fdUCol4:

        rol     eax,16
                        
        mov     [edi],eax
        add     edi,4
        mov     ebx,edx
        shr     ebx,32 - SOURCE_SBITS
        shld    ebx,ecx,SOURCE_SBITS
        add     edx,FIXUP
@@fdVCol5:
        add     ecx,FIXUP
@@fdUCol5:
        mov     al,[esi+ebx]        
        mov     ebx,edx
        shr     ebx,32 - SOURCE_SBITS
        shld    ebx,ecx,SOURCE_SBITS
        add     edx,FIXUP
@@fdVCol6:
        add     ecx,FIXUP
@@fdUCol6:
        mov     ah,[esi+ebx]

        mov     ebx,edx
        shl     eax,16
        
        shr     ebx,32 - SOURCE_SBITS
        shld    ebx,ecx,SOURCE_SBITS
        add     edx,FIXUP
@@fdVCol7:
        mov     al,[esi+ebx]
        add     ecx,FIXUP
@@fdUCol7:

        mov     ebx,edx
        
        shr     ebx,32 - SOURCE_SBITS
        shld    ebx,ecx,SOURCE_SBITS
        add     edx,FIXUP
@@fdVCol8:
        mov     ah,[esi+ebx]
        add     ecx,FIXUP
@@fdUCol8:

        rol     eax,16
                        
        mov     [edi],eax
        add     edi,4
        dec     ebp
                
        jnz     @@colloop
;----------------------------------------------
        add     ecx,FIXUP
@@fdURow:
        add     edx,FIXUP
@@fdVRow:

        pop     ebp
        dec     [rowCount]
        jnz     @@rowloop
;----------------------------------------------        
        popa
        ret

RotateBlit_                 ENDP
;------------------------------------------------------------------
;------------------------------------------------------------------
;       AddBlit
;
;       Remaps colors of the screen buffer ( new value=old value+128 ) 
;       input: edi=screen buffer (320x240)
;       output: none
;              
;------------------------------------------------------------------

AddBlit_        Proc    NEAR
        
        mov     ecx,(SCREEN_WIDTH*SCREEN_HEIGHT)/32
        
loopik:        
        or      [edi],80808080h        ; unroll the loop ( 32 pixels at a time )                 
        add     edi,4
        or      [edi],80808080h        
        add     edi,4        
        or      [edi],80808080h        
        add     edi,4
        or      [edi],80808080h        
        add     edi,4
        or      [edi],80808080h        
        add     edi,4
        or      [edi],80808080h        
        add     edi,4        
        or      [edi],80808080h        
        add     edi,4
        or      [edi],80808080h        
        add     edi,4
                
        dec     ecx
        jnz     loopik
        ret
AddBlit_        Endp
;------------------------------------------------------------------
;------------------------------------------------------------------
;       SubBlit
;
;       Displays semi-trasparent logo  
;       input: edi=screen buffer (320x240), esi=logo bitmap
;       output: none
;              
;------------------------------------------------------------------

SubBlit_        PROC NEAR
        
        add     edi,204*SCREEN_WIDTH+114
        mov     ecx,32
pel2:        
        mov     edx,200/8
      
pel1:        
        mov     eax,[esi]                            
        and     [edi],eax
        add     edi,4
        add     esi,4
        mov     eax,[esi]
        and     [edi],eax
        add     edi,4
        add     esi,4        
        dec     edx
        jnz     pel1
      
        add     edi,SCREEN_WIDTH-200
        dec     ecx
        jnz     pel2

        ret
SubBlit_        endp
      

_TEXT   ENDS
        
        END

 
