
//***************************************************************
//*  SVGAdemo.c  - Scaling and rotating in SVGA
//*  (WATCOM C 10.5 + WASM ) 
//***************************************************************

         #include <stdio.h>
         #include <stdlib.h>
         #include <mem.h>
         #include <math.h> 
         #include <dos.h>
         #include <string.h>
         #include <stdlib.h>
         #include <malloc.h>
         #include <direct.h>    
         #include <bios.h>
         #include <conio.h> 

//***************************************************************
//* Type definitions and defines
//***************************************************************

         typedef signed short   WORD;
         typedef unsigned char  UBYTE;
         typedef unsigned short UWORD;
         typedef unsigned long  ULONG;

         #define kAngle360 256
         #define kAngle180 128
         #define kPi 3.141592654 

//  Functions
       
         void SetVMode(UWORD mode);                      
         UWORD SetSVGA ();                       
         UWORD CheckSVGA ();        
         void ClearSVGA (UBYTE color);           
         void SetPage (UBYTE page);
         void LoadBINBitmap(char *filename,UBYTE *buffer,ULONG size);
         void SetPalette(UBYTE *palette);
         void InitSinTable();

// WASM - external functions      
         extern void near  RotateBlit(UBYTE *bitmap, int angle, long scale,UBYTE *vbuffor);
         #pragma aux RotateBlit "*_" parm caller [esi] [eax] [edx] [edi];
         extern void near  AddBlit(UBYTE *vbuffor);
         #pragma aux AddBlit "*_" parm caller [edi] [edx];
         extern void near  SubBlit(UBYTE *vbuffor,UBYTE *bitmap);
         #pragma aux SubBlit "*_" parm caller [edi] [esi];
         extern void near BlitMemVram(UBYTE *data,UWORD x,UWORD y,UWORD width,
                                      UWORD height);
         #pragma aux BlitMemVram "*_" parm caller [esi] [ebx] [eax] [ecx] [edi] ;
                      
      
//****************************************************************
// Global variables
//****************************************************************
         
         long sinTable[kAngle360];   

         ULONG *spointer;                // sinTable pointer (WASM)
         UWORD Addgran=0;                // page granularity
         UWORD Curwind=0;                // current page #

//****************************************************************
//****************************************************************


     void main()
        {
          
         char *fname = "malen.bin";   // load bitmaps 
         char *fname2= "malen2.bin";
         char *cname = "male.col";    // load palette
  
         UBYTE *text_buffer;
         UBYTE *bitmap_buffer;
         UBYTE *screen_buffer;
         UBYTE palette[768];
  
         int frames=0;
         int angle;
         ULONG time_start,time_end;
         UWORD fps10;         
// allocate memory
   
         screen_buffer=malloc(87300);         
         if(!screen_buffer)
         {
          return;
         }
// initialize pointers
         
         bitmap_buffer=screen_buffer+76800;
         text_buffer=bitmap_buffer+4096;
// prepare look-up table for sin & cos 
  
         InitSinTable();
         spointer=&sinTable;
// load bitmaps and palette ( if some files are missing then the program displays
//  whatever it is in memory -> you'll see scaled and rotated garbage ) 
        
         LoadBINBitmap(fname,bitmap_buffer,64*64);
         LoadBINBitmap(fname2,text_buffer,200*32);
         LoadBINBitmap(cname,&palette,768);
// look for VESA BIOS

         if (CheckSVGA())
          return ;
// set mode 640x480x256
         fflush(stdout);
         delay (600);
         SetSVGA();
         ClearSVGA(0);         
// initialize palette
        
         SetPalette(&palette);
// main loop         
         frames=0;  
         _bios_timeofday(_TIME_GETCLOCK,&time_start);   // get DOS ticks
           
  
         while ( 1 )
         {
          if ( kbhit() )
          break;
    
          for (angle = 255; angle >-1; angle=angle-2)
          {
           RotateBlit(bitmap_buffer, angle,
           sinTable[angle]/2 + 0x9000,screen_buffer);   // rotate and scale bitmap
           
           BlitMemVram(screen_buffer,320,0,320,240);    // display two bitmaps on the screen   
           BlitMemVram(screen_buffer,0,240,320,240);    // ( blue )
           
           AddBlit(screen_buffer);                    // quickly change colors   
             
           BlitMemVram(screen_buffer,0,0,320,240);      // now display our bitmap 
                                                        // in new colors (red) 
           SubBlit(screen_buffer,text_buffer);          // add logo to the bitmap    
           BlitMemVram(screen_buffer,320,240,320,240);  // and display it on the screen 
    
           frames++;                                    // count frames  
          }
         }
    
         _bios_timeofday(_TIME_GETCLOCK,&time_end);     // get DOS ticks again  
// free buffer and restore text mode 3
                  
         free(screen_buffer);                           
         SetVMode(3);
// compute average FPS and display it
         
         fps10=frames*182/(time_end-time_start);                              
         printf("Frame rate: %i.%i fps\n", fps10/10,fps10%10);
  
         return;
        }



//**************************************************************
//  SetPalette - Sets DAC                                      *
//**************************************************************

        void    SetPalette(UBYTE *palette)
        {
         UWORD   i;
         UBYTE *p;
            
         p=palette;
         outp(0x3c6,0xff);
         for(i=0;i<=255;i++)
         {
          outp(0x3c8,(UBYTE)i);
          outp(0x3c9,*p++);
          outp(0x3c9,*p++);
          outp(0x3c9,*p++);
         }            
         return ;
        }
//**************************************************************
//  LoadBINBitmap - loads bitmap (bin)                         *
//**************************************************************

        void LoadBINBitmap(char *filename,UBYTE *buffer,ULONG size)
        { 
         FILE *fp;

         fp=fopen (filename,"rb");
         if(!fp)
         {
          return;
         }
         fread(buffer,1,size,fp);
         fclose(fp);
       
         return;
        }

//**************************************************************
//  InitSinTable - prepares sin-cos look-up table              *
//**************************************************************
        
        void InitSinTable()
        {
         int i;

         for (i = 0; i < kAngle180; i++)
         sinTable[i] = sin((double)i * kPi / kAngle180) * 0x10000L; //fixed point values

         for (i = kAngle180; i < kAngle360; i++)
         sinTable[i] = -sinTable[i - kAngle180];
        }
        
//**************************************************************
//  SetVMode - sets screen mode (VGA only)                     *
//**************************************************************
      
        void SetVMode (UWORD mode)
        {          
         union REGS regs;
         regs.w.ax=mode;
         int386 (0x10,&regs,&regs);          
         return;
        }


//*********************************************************************                          
//  CheckSVGA - checks for SVGA 640x480 VESA mode, error=1 if not found 
//*********************************************************************
      
        UWORD CheckSVGA()
        
        {
         static struct rminfo
         {        
          long EDI;
          long ESI;
          long EBP;
          long reserved_by_system;
          long EBX;
          long EDX;
          long ECX;
          long EAX;
          short flags;
          short ES,DS,FS,GS,IP,CS,SP,SS;
         } RMI;

     
         union  REGS regs;
         struct SREGS sregs;
         
         short selector;
         char  far *point;              
         short segment;
         UWORD error=0;
         
// DPMI call 100h allocates DOS memory for SVGA info buffer

         memset(&sregs,0,sizeof(sregs));
         regs.w.ax=0x0100;
         regs.w.bx=0x0010;                 
         int386x( 0x31, &regs, &regs, &sregs);
         segment=regs.w.ax;
         selector=regs.w.dx;

         point=MK_FP(selector,0);
               

// call VESA service 4F00 - return SVGA information

         memset(&RMI,0,sizeof(RMI));
         RMI.EAX=0x00004f00;              // service #
         RMI.ES=segment;                  // segment 
         RMI.EDI=0;                       // offset
  
         regs.w.ax = 0x0300;              
         regs.h.bl = 0x10;
         regs.h.bh = 0;
         regs.w.cx = 0;
         sregs.es = FP_SEG(&RMI);
         regs.x.edi = FP_OFF(&RMI);
         int386x( 0x31, &regs, &regs, &sregs );     // use DMPI to isue DOS INT

         printf ("\n==========================\n");
                   
// IF not succesful - print message and exit
        
         if (!(RMI.EAX==0x004f))
         {                      
          printf("ERROR : VESA BIOS not installed" );
          error=1;             
          goto L1;
         }
         
// ELSE print information about SVGA
            
         else
         {         
          printf ("Vesa bios found :version %c.%c",(*(point+5))+48,(*(point+4))+48);
          printf ("\n%d Kb video ram",(*(point+18))*64);
         }                  
         
// call VESA service 4F01 - return mode information

         memset(&RMI,0,sizeof(RMI));
         RMI.EAX=0x00004f01;              
         RMI.ES=segment;
         RMI.EDI=0;                       
         RMI.ECX=0x00000101;                
  
         regs.w.ax = 0x0300;              
         regs.h.bl = 0x10;
         regs.h.bh = 0;
         regs.w.cx = 0;
         sregs.es = FP_SEG(&RMI);
         regs.x.edi = FP_OFF(&RMI);
         int386x( 0x31, &regs, &regs, &sregs );          

         if (!(RMI.EAX==0x004f))
         {                      
          printf("\nERROR : Mode 640x480x8 not supported" );
          error=1;  
          goto L1;
         }            
         else
         {                  
          printf("\nMode 640x480x8 supported");            
          Addgran=64/(*(point+4));
          printf("\nWindow granularity =%d Kb",*(point+4));         
         }
         
// DPMI call 100h deallocates DOS memory used by SVGA info buffer

  L1:;
         regs.w.ax=0x0101;
         regs.w.dx=selector; 
         int386( 0x31, &regs, &regs);
                      
         return(error); 
        }

//*************************************************************                          
//  SetSVGA - sets 640x480 SVGA mode                          * 
//*************************************************************

         UWORD SetSVGA ()              
        {
      
         union REGS regs;
         
         regs.w.ax=0x4f02;
         regs.w.bx=0x101;    
         int386( 0x10, &regs, &regs);

         if (!(regs.w.ax==0x004f))
          {                      
             return (1);
          }         

         SetPage (0);    
         return(0);   
          
        }

//*************************************************************                          
//  ClearSVGA - clears (fills) screen                         * 
//*************************************************************        
        
        void   ClearSvga (UBYTE color)        
        {
         UBYTE j;
          
         for (j=0; j<4; j++)
         {   
          SetPage(j);
          memset(0xA0000,color,65536);
         }              
         SetPage(4);
         memset (0xA0000,color,45056);
            
         return ;
        }

//*************************************************************                          
//  SetPage - sets page using VESA BIOS call                  * 
//*************************************************************        
        
        void   SetPage (UBYTE page)        
        {
         union REGS regs;

         Curwind=page;  
         page=Addgran*page;                // page#=granularity * page# 
         regs.w.ax=0x4f05;
         regs.w.bx=0x000;                
         regs.h.dl=page;                  
         int386( 0x10, &regs, &regs);      
            
         return;
        }



